@@ -169,13 +169,13 @@ int qat_fips_kat_test;
 const char *engine_qat_id = STR(QAT_ENGINE_ID);
 #if defined(QAT_HW) && defined(QAT_SW)
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_hw & qat_sw) v1.8.1";
+    "Reference implementation of QAT crypto engine(qat_hw & qat_sw) v2.0.0";
 #elif QAT_HW
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_hw) v1.8.1";
+    "Reference implementation of QAT crypto engine(qat_hw) v2.0.0";
 #else
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_sw) v1.8.1";
+    "Reference implementation of QAT crypto engine(qat_sw) v2.0.0";
 #endif
 unsigned int engine_inited = 0;
 int fallback_to_openssl = 0;
@@ -1117,7 +1117,7 @@ int bind_qat(ENGINE *e, const char *id)
    int ret = 0;
 #ifdef QAT_HW
     char *config_section = NULL;
-# if !defined(QAT_HW_INTREE) && (defined(QAT20_OOT) || defined(__FreeBSD__))
+#ifdef ENABLE_QAT_HW_KPT
     Cpa32U dev_count = 0;
 # endif
 #endif
@@ -1132,7 +1132,7 @@ int bind_qat(ENGINE *e, const char *id)
 
     /* For QAT_HW, Check if the QAT_HW device is available */
 #ifdef QAT_HW
-# if !defined(QAT_HW_INTREE) && (defined(QAT20_OOT) || defined(__FreeBSD__))
+# ifdef ENABLE_QAT_HW_KPT
     if (icp_adf_get_numDevices(&dev_count) == CPA_STATUS_SUCCESS) {
         if (dev_count > 0) {
             qat_hw_offload = 1;
@@ -1144,10 +1144,15 @@ int bind_qat(ENGINE *e, const char *id)
         qat_hw_offload = 1;
         DEBUG("QAT HW device available\n");
     }
-#endif
+# endif
     if (!qat_hw_offload) {
 # ifndef QAT_SW
+#  ifdef QAT_BORINGSSL
+        fprintf(stderr, "QAT_HW device not available & QAT_SW not enabled. Exiting!\n");
+        return ret;
+#  else
         fprintf(stderr, "QAT_HW device not available & QAT_SW not enabled. Using OpenSSL_SW!\n");
+#  endif
 # endif
     }
 #endif
@@ -1298,6 +1303,10 @@ int bind_qat(ENGINE *e, const char *id)
             qat_hw_gcm_offload = 1;
             DEBUG("QAT_HW GCM for Provider Enabled\n");
         }
+# endif
+# ifdef ENABLE_QAT_HW_CIPHERS
+        qat_hw_aes_cbc_hmac_sha_offload = 1;
+        INFO("QAT_HW CIPHERS for Provider Enabled\n");
 # endif
     }
 
@@ -1314,8 +1323,8 @@ int bind_qat(ENGINE *e, const char *id)
 
 # ifdef ENABLE_QAT_SW_ECDSA
         if (!qat_hw_ecdsa_offload &&
-            mbx_get_algo_info(MBX_ALGO_ECDSA_NIST_P256) &&
-            mbx_get_algo_info(MBX_ALGO_ECDSA_NIST_P384)) {
+            (mbx_get_algo_info(MBX_ALGO_ECDSA_NIST_P256) ||
+             mbx_get_algo_info(MBX_ALGO_ECDSA_NIST_P384))) {
             qat_sw_ecdsa_offload = 1;
             INFO("QAT_SW ECDSA for Provider Enabled\n");
         }
@@ -1323,8 +1332,8 @@ int bind_qat(ENGINE *e, const char *id)
 
 # ifdef ENABLE_QAT_SW_ECDH
         if (!qat_hw_ecdh_offload &&
-            mbx_get_algo_info(MBX_ALGO_ECDHE_NIST_P256) &&
-            mbx_get_algo_info(MBX_ALGO_ECDHE_NIST_P384)) {
+            (mbx_get_algo_info(MBX_ALGO_ECDHE_NIST_P256) ||
+            mbx_get_algo_info(MBX_ALGO_ECDHE_NIST_P384))) {
             qat_sw_ecdh_offload = 1;
             INFO("QAT_SW ECDH for Provider Enabled\n");
         }
