@@ -1,24 +1,20 @@
 /*
- * Copyright 1995-2024 The OpenSSL Project Authors. All Rights Reserved.
- *
- * Licensed under the OpenSSL license (the "License").  You may not use
+ * Licensed under the Apache License 2.0 (the "License").  You may not use
  * this file except in compliance with the License.  You can obtain a copy
  * in the file LICENSE in the source distribution or at
  * https://www.openssl.org/source/license.html
  */
 
+
 #include <openssl/err.h>
 #include "qat_bssl_err.h"
 
-#define  OPENSSL_NO_ERR
 #ifndef OPENSSL_NO_ERR
 
 static ERR_STRING_DATA QAT_str_functs[] = {
     {ERR_PACK(0, QAT_F_AES_GCM_TLS_CIPHER, 0), "aes_gcm_tls_cipher"},
     {ERR_PACK(0, QAT_F_BUILD_DECRYPT_OP_BUF, 0), "build_decrypt_op_buf"},
     {ERR_PACK(0, QAT_F_BUILD_ENCRYPT_OP_BUF, 0), "build_encrypt_op_buf"},
-    {ERR_PACK(0, QAT_F_CRT_COMBINE, 0), "CRT_combine"},
-    {ERR_PACK(0, QAT_F_CRT_PREPARE, 0), "CRT_prepare"},
     {ERR_PACK(0, QAT_F_ENGINE_FINISH_BEFORE_FORK_HANDLER, 0),
      "engine_finish_before_fork_handler"},
     {ERR_PACK(0, QAT_F_ENGINE_INIT_CHILD_AT_FORK_HANDLER, 0),
@@ -48,8 +44,6 @@ static ERR_STRING_DATA QAT_str_functs[] = {
      "multibuff_validate_ecx_derive"},
     {ERR_PACK(0, QAT_F_MULTIBUFF_X25519_DERIVE, 0), "multibuff_x25519_derive"},
     {ERR_PACK(0, QAT_F_MULTIBUFF_X25519_KEYGEN, 0), "multibuff_x25519_keygen"},
-    {ERR_PACK(0, QAT_F_OSSL_SM2_COMPUTE_Z_DIGEST, 0),
-     "ossl_sm2_compute_z_digest"},
     {ERR_PACK(0, QAT_F_POLL_INSTANCES, 0), "poll_instances"},
     {ERR_PACK(0, QAT_F_QAT_ADJUST_THREAD_AFFINITY, 0),
      "qat_adjust_thread_affinity"},
@@ -117,6 +111,8 @@ static ERR_STRING_DATA QAT_str_functs[] = {
     {ERR_PACK(0, QAT_F_QAT_HW_FINISH_INT, 0), "qat_hw_finish_int"},
     {ERR_PACK(0, QAT_F_QAT_HW_INIT, 0), "qat_hw_init"},
     {ERR_PACK(0, QAT_F_QAT_HW_SHA3_OFFLOAD, 0), "qat_hw_sha3_offload"},
+    {ERR_PACK(0, QAT_F_QAT_HW_SM2_COMPUTE_Z_DIGEST, 0),
+     "qat_hw_sm2_compute_z_digest"},
     {ERR_PACK(0, QAT_F_QAT_HW_SM3_CLEANUP, 0), "qat_hw_sm3_cleanup"},
     {ERR_PACK(0, QAT_F_QAT_HW_SM3_COPY, 0), "qat_hw_sm3_copy"},
     {ERR_PACK(0, QAT_F_QAT_HW_SM3_DO_OFFLOAD, 0), "qat_hw_sm3_do_offload"},
@@ -136,7 +132,6 @@ static ERR_STRING_DATA QAT_str_functs[] = {
     {ERR_PACK(0, QAT_F_QAT_PRF_TLS_DERIVE, 0), "qat_prf_tls_derive"},
     {ERR_PACK(0, QAT_F_QAT_REMAP_INSTANCES, 0), "qat_remap_instances"},
     {ERR_PACK(0, QAT_F_QAT_RSA_DECRYPT, 0), "qat_rsa_decrypt"},
-    {ERR_PACK(0, QAT_F_QAT_RSA_DECRYPT_CRT, 0), "qat_rsa_decrypt_CRT"},
     {ERR_PACK(0, QAT_F_QAT_RSA_ENCRYPT, 0), "qat_rsa_encrypt"},
     {ERR_PACK(0, QAT_F_QAT_RSA_PRIV_DEC, 0), "qat_rsa_priv_dec"},
     {ERR_PACK(0, QAT_F_QAT_RSA_PRIV_ENC, 0), "qat_rsa_priv_enc"},
@@ -186,7 +181,6 @@ static ERR_STRING_DATA QAT_str_functs[] = {
     {ERR_PACK(0, QAT_F_QAT_SW_SM4_GCM_INIT, 0), "qat_sw_sm4_gcm_init"},
     {ERR_PACK(0, QAT_F_QAT_SW_SM4_GCM_TLS_CIPHER, 0),
      "qat_sw_sm4_gcm_tls_cipher"},
-    {ERR_PACK(0, QAT_F_QAT_SYM_PERFORM_OP, 0), "qat_sym_perform_op"},
     {ERR_PACK(0, QAT_F_QAT_VALIDATE_ECX_DERIVE, 0), "qat_validate_ecx_derive"},
     {ERR_PACK(0, QAT_F_QAT_X25519_PMETH, 0), "qat_x25519_pmeth"},
     {ERR_PACK(0, QAT_F_QAT_X448_PMETH, 0), "qat_x448_pmeth"},
@@ -224,6 +218,8 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     "alloc qat x448 meth failure"},
     {ERR_PACK(0, 0, QAT_R_ALLOC_TAG_FAILURE), "alloc tag failure"},
     {ERR_PACK(0, 0, QAT_R_BAD_INPUT_PARAMS), "bad input params"},
+    {ERR_PACK(0, 0, QAT_R_BAD_LENGTH), "bad length"},
+    {ERR_PACK(0, 0, QAT_R_BAD_TLS_CLIENT_VERSION), "bad tls client version"},
     {ERR_PACK(0, 0, QAT_R_BN_LIB_FAILURE), "bn lib failure"},
     {ERR_PACK(0, 0, QAT_R_BUFFER_TOO_SMALL), "buffer too small"},
     {ERR_PACK(0, 0, QAT_R_BUF_CONV_FAIL), "buf conv fail"},
@@ -250,6 +246,8 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_C_MODULO_Q_FAILURE), "c modulo q failure"},
     {ERR_PACK(0, 0, QAT_R_C_P_Q_CP_CQ_MALLOC_FAILURE),
     "c p q cp cq malloc failure"},
+    {ERR_PACK(0, 0, QAT_R_DATA_TOO_LARGE_FOR_KEY_SIZE),
+    "data too large for key size"},
     {ERR_PACK(0, 0, QAT_R_DEC_OP_DATA_MALLOC_FAILURE),
     "dec op data malloc failure"},
     {ERR_PACK(0, 0, QAT_R_DERIVE_FAILURE), "derive failure"},
@@ -297,6 +295,7 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_EPOLL_CTL_FAILURE), "epoll ctl failure"},
     {ERR_PACK(0, 0, QAT_R_EVENTS_MALLOC_FAILURE), "events malloc failure"},
     {ERR_PACK(0, 0, QAT_R_EVP_LIB), "evp lib"},
+    {ERR_PACK(0, 0, QAT_R_FAILED_TO_DECRYPT), "failed to decrypt"},
     {ERR_PACK(0, 0, QAT_R_FAILED_TO_GET_PARAMETER), "failed to get parameter"},
     {ERR_PACK(0, 0, QAT_R_FAILED_TO_SET_PARAMETER), "failed to set parameter"},
     {ERR_PACK(0, 0, QAT_R_FALLBACK_INIT_FAILURE), "fallback init failure"},
@@ -330,6 +329,7 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_ICP_SAL_USERSTART_FAIL), "icp sal userstart fail"},
     {ERR_PACK(0, 0, QAT_R_ID_TOO_LARGE), "id too large"},
     {ERR_PACK(0, 0, QAT_R_INITIALIZE_CTX_FAILURE), "initialize ctx failure"},
+    {ERR_PACK(0, 0, QAT_R_INIT_FAIL), "init fail"},
     {ERR_PACK(0, 0, QAT_R_INIT_FAILURE), "init failure"},
     {ERR_PACK(0, 0, QAT_R_INPUT_DATA_MALLOC_FAILURE),
     "input data malloc failure"},
@@ -343,15 +343,18 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_INVALID_CTRL_TYPE), "invalid ctrl type"},
     {ERR_PACK(0, 0, QAT_R_INVALID_CURVE), "invalid curve"},
     {ERR_PACK(0, 0, QAT_R_INVALID_DATA), "invalid data"},
+    {ERR_PACK(0, 0, QAT_R_INVALID_DIGEST), "invalid digest"},
     {ERR_PACK(0, 0, QAT_R_INVALID_HASH_DATA), "invalid hash data"},
     {ERR_PACK(0, 0, QAT_R_INVALID_INPUT), "invalid input"},
     {ERR_PACK(0, 0, QAT_R_INVALID_INPUT_LENGTH), "invalid input length"},
     {ERR_PACK(0, 0, QAT_R_INVALID_INPUT_PARAMETER), "invalid input parameter"},
     {ERR_PACK(0, 0, QAT_R_INVALID_IVLEN), "invalid ivlen"},
     {ERR_PACK(0, 0, QAT_R_INVALID_IV_LENGTH), "invalid iv length"},
+    {ERR_PACK(0, 0, QAT_R_INVALID_KEY), "invalid key"},
     {ERR_PACK(0, 0, QAT_R_INVALID_KEY_LENGTH), "invalid key length"},
     {ERR_PACK(0, 0, QAT_R_INVALID_L), "invalid l"},
     {ERR_PACK(0, 0, QAT_R_INVALID_LEN), "invalid len"},
+    {ERR_PACK(0, 0, QAT_R_INVALID_LENGTH), "invalid length"},
     {ERR_PACK(0, 0, QAT_R_INVALID_PEER_KEY), "invalid peer key"},
     {ERR_PACK(0, 0, QAT_R_INVALID_PRIVATE_KEY), "invalid private key"},
     {ERR_PACK(0, 0, QAT_R_INVALID_PTR), "invalid ptr"},
@@ -380,6 +383,7 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_KEY_MALLOC_FAILURE), "key malloc failure"},
     {ERR_PACK(0, 0, QAT_R_KEY_NOTSET), "key notset"},
     {ERR_PACK(0, 0, QAT_R_KEY_NULL), "key null"},
+    {ERR_PACK(0, 0, QAT_R_KEY_SIZE_TOO_SMALL), "key size too small"},
     {ERR_PACK(0, 0, QAT_R_K_ALLOCATE_FAILURE), "k allocate failure"},
     {ERR_PACK(0, 0, QAT_R_K_CONVERT_TO_FB_FAILURE), "k convert to fb failure"},
     {ERR_PACK(0, 0, QAT_R_K_E_ORDER_MALLOC_FAILURE),
@@ -427,6 +431,8 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_OPDCRT_NULL), "opdcrt null"},
     {ERR_PACK(0, 0, QAT_R_OPDONE_NULL), "opdone null"},
     {ERR_PACK(0, 0, QAT_R_OPDPIPE_NULL), "opdpipe null"},
+    {ERR_PACK(0, 0, QAT_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE),
+    "operation not supported for this keytype"},
     {ERR_PACK(0, 0, QAT_R_ORDER_MALLOC_FAILURE), "order malloc failure"},
     {ERR_PACK(0, 0, QAT_R_OUT1_PDATA_MALLOC_FAILURE),
     "out1 pdata malloc failure"},
@@ -441,6 +447,7 @@ static ERR_STRING_DATA QAT_str_reasons[] = {
     {ERR_PACK(0, 0, QAT_R_OUTX_OUTY_LEN_NULL), "outx outy len null"},
     {ERR_PACK(0, 0, QAT_R_OUTY_MALLOC_FAILURE), "outy malloc failure"},
     {ERR_PACK(0, 0, QAT_R_PADDING_UNKNOWN), "padding unknown"},
+    {ERR_PACK(0, 0, QAT_R_PKCS_DECODING_ERROR), "pkcs decoding error"},
     {ERR_PACK(0, 0, QAT_R_POLLING_THREAD_CREATE_FAILURE),
     "polling thread create failure"},
     {ERR_PACK(0, 0, QAT_R_POLLING_THREAD_SEM_INIT_FAILURE),
