@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2024 Intel Corporation.
+ *   Copyright(c) 2016-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -48,7 +48,9 @@
 
 # ifdef QAT_HW
 
-# include <openssl/engine.h>
+# ifndef QAT_OPENSSL_PROVIDER
+#  include <openssl/engine.h>
+# endif
 # include <openssl/ssl.h>
 # include <openssl/crypto.h>
 # include <openssl/aes.h>
@@ -189,6 +191,7 @@ int qat_chained_ciphers_init(PROV_CIPHER_CTX *ctx,
                              const unsigned char *iv, size_t ivlen, int enc);
 int qat_chained_ciphers_cleanup(PROV_CIPHER_CTX *ctx);
 int qat_chained_ciphers_do_cipher(PROV_CIPHER_CTX *ctx, unsigned char *out,
+                                  size_t *outl, size_t outsize,
                                   const unsigned char *in, size_t len);
 int qat_chained_ciphers_ctrl(PROV_AES_HMAC_SHA_CTX *ctx, int type, int arg, void *ptr);
 # else
