@@ -68,7 +68,7 @@ typedef struct evp_signature_st {
     char *type_name;
     const char *description;
     OSSL_PROVIDER *prov;
-    CRYPTO_REF_COUNT references;
+    QAT_CRYPTO_REF_COUNT references;
 #if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_RWLOCK *lock;
 #endif
@@ -1016,6 +1016,7 @@ int qat_sm2_sign(EVP_PKEY_CTX *ctx,
 #  endif
 # endif
     }
+    ECDSA_SIG_free(s);
     DEBUG("- FinishedP: %d\n", ret);
     return ret;
 }
@@ -1449,6 +1450,8 @@ int qat_sm2_verify(EVP_PKEY_CTX *ctx,
 #  endif
 # endif
     }
+    ECDSA_SIG_free(s);
+    OPENSSL_free(dgst);
     DEBUG("- Finished\n");
     return ret;
 }
