@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023-2024 Intel Corporation.
+ *   Copyright(c) 2023-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,10 @@
 #ifndef QAT_HW_SM3_H
 # define QAT_HW_SM3_H
 
+#ifndef QAT_OPENSSL_PROVIDER
 # include <openssl/engine.h>
+#endif
+# include <openssl/evp.h>
 
 # include "cpa.h"
 # include "cpa_types.h"
