@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2024 Intel Corporation.
+ *   Copyright(c) 2024-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -93,7 +93,6 @@ const char *qat_ccm_cipher_name(int nid)
     }
 }
 
-#if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE)) || !defined(ENABLE_QAT_SMALL_PKT_OFFLOAD)
 QAT_EVP_CIPHER get_default_cipher_aes_ccm(int nid)
 {
     static QAT_EVP_CIPHER ccm_cipher;
@@ -112,7 +111,37 @@ QAT_EVP_CIPHER get_default_cipher_aes_ccm(int nid)
     }
     return ccm_cipher;
 }
-#endif
+
+static int qat_aes_ccm_tls_init(QAT_PROV_CCM_CTX *ctx, unsigned char *aad, size_t alen)
+{
+    size_t len;
+
+    if (!qat_prov_is_running() || alen != EVP_AEAD_TLS1_AAD_LEN)
+        return 0;
+
+    /* Save the aad for later use. */
+    memcpy(ctx->buf, aad, alen);
+    ctx->tls_aad_len = alen;
+
+    len = ctx->buf[alen - 2] << 8 | ctx->buf[alen - 1];
+    if (len < EVP_CCM_TLS_EXPLICIT_IV_LEN)
+        return 0;
+
+    /* Correct length for explicit iv. */
+    len -= EVP_CCM_TLS_EXPLICIT_IV_LEN;
+
+    if (!ctx->enc) {
+        if (len < ctx->M)
+            return 0;
+        /* Correct length for tag. */
+        len -= ctx->M;
+    }
+    ctx->buf[alen - 2] = (unsigned char)(len >> 8);
+    ctx->buf[alen - 1] = (unsigned char)(len & 0xff);
+
+    /* Extra padding: tag appended to record. */
+    return ctx->M;
+}
 
 static void *qat_aes_ccm_newctx(void *provctx, size_t keybits, int nid)
 {
@@ -141,61 +170,99 @@ size_t qat_aes_ccm_get_ivlen(QAT_PROV_CCM_CTX * ctx)
     return QAT_AES_CCM_OP_VALUE - ctx->L;
 }
 
-int qat_aes_ccm_einit(void *ctx, const unsigned char *inkey, size_t keylen,
-                      const unsigned char *iv, size_t ivlen, int enc)
+int qat_aes_ccm_einit(void *vctx, const unsigned char *inkey, size_t keylen,
+                      const unsigned char *iv, size_t ivlen, const OSSL_PARAM param[])
 {
     int sts = 0;
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
+    QAT_PROV_CCM_CTX *ctx = (QAT_PROV_CCM_CTX *) vctx;
+    ctx->enc = 1;
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
     QAT_PROV_CCM_CTX *qctx = (QAT_PROV_CCM_CTX *) ctx;
+#  ifdef QAT_INSECURE_ALGO
+    if (qctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    QAT_PROV_CCM_CTX *qctx = (QAT_PROV_CCM_CTX *) ctx;
+    if (qctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
+# endif
+    if (qat_hw_aes_ccm_offload) {
+        sts = qat_aes_ccm_init(ctx, inkey, keylen, iv, ivlen, 1);
+        if (sts != 1) {
+            QATerr(ERR_LIB_PROV, QAT_R_EINIT_OPERATION_FAILED);
+            return sts;
+        }
+    }
+
+    return qat_aes_ccm_set_ctx_params(ctx, param);
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
     OSSL_PARAM params[2] = { OSSL_PARAM_END, OSSL_PARAM_END };
     QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(qctx->nid);
-
-    if (qctx->nid == NID_aes_192_ccm || qctx->nid == NID_aes_256_ccm) {
-        if (!qctx->sw_ctx)
-            qctx->sw_ctx = sw_aes_ccm_cipher.newctx(ctx);
-
-        sts =
-            sw_aes_ccm_cipher.einit(qctx->sw_ctx, inkey, keylen, iv, ivlen,
-                                    params);
-        if (sts != 1)
-            return 0;
-
-        return sts;
-    }
-# endif
-    if (qat_hw_aes_ccm_offload)
-        sts = qat_aes_ccm_init(ctx, inkey, keylen, iv, ivlen, 1);
-
-    return sts;
+    if (qctx->sw_ctx || (qctx->sw_ctx = sw_aes_ccm_cipher.newctx(ctx)))
+        return sw_aes_ccm_cipher.einit(qctx->sw_ctx, inkey, keylen, iv, ivlen,
+			                             params);
+    return 0;
+#endif
 }
 
-int qat_aes_ccm_dinit(void *ctx, const unsigned char *inkey, size_t keylen,
-                      const unsigned char *iv, size_t ivlen, int enc)
+int qat_aes_ccm_dinit(void *vctx, const unsigned char *inkey, size_t keylen,
+                      const unsigned char *iv, size_t ivlen, const OSSL_PARAM param[])
 {
     int sts = 0;
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
+    QAT_PROV_CCM_CTX *ctx = (QAT_PROV_CCM_CTX *) vctx;
+    ctx->enc = 0;
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
     QAT_PROV_CCM_CTX *qctx = (QAT_PROV_CCM_CTX *) ctx;
+#  ifdef QAT_INSECURE_ALGO
+    if (qctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    QAT_PROV_CCM_CTX *qctx = (QAT_PROV_CCM_CTX *) ctx;
+    if (qctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
+# endif
+    if (qat_hw_aes_ccm_offload) {
+        sts = qat_aes_ccm_init(ctx, inkey, keylen, iv, ivlen, 0);
+        if (sts != 1) {
+            QATerr(ERR_LIB_PROV, QAT_R_DINIT_OPERATION_FAILED);
+            return sts;
+        }
+    }
+
+    return qat_aes_ccm_set_ctx_params(ctx, param);
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
     OSSL_PARAM params[2] = { OSSL_PARAM_END, OSSL_PARAM_END };
     QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(qctx->nid);
-    if (qctx->nid == NID_aes_192_ccm || qctx->nid == NID_aes_256_ccm) {
-        if (!qctx->sw_ctx)
-            qctx->sw_ctx = sw_aes_ccm_cipher.newctx(ctx);
-
-        unsigned int pad = 0;
-        params[0] = OSSL_PARAM_construct_uint(OSSL_CIPHER_PARAM_PADDING, &pad);
-        sts =
+    if (!qctx->sw_ctx)
+         qctx->sw_ctx = sw_aes_ccm_cipher.newctx(ctx);
+    unsigned int pad = 0;
+    params[0] = OSSL_PARAM_construct_uint(OSSL_CIPHER_PARAM_PADDING, &pad);
+    sts =
             sw_aes_ccm_cipher.dinit(qctx->sw_ctx, inkey, keylen, iv, ivlen,
                                     params);
-        if (sts != 1)
+    if (sts != 1)
             return 0;
 
-        return sts;
-    }
-# endif
-    if (qat_hw_aes_ccm_offload)
-        sts = qat_aes_ccm_init(ctx, inkey, keylen, iv, ivlen, 0);
-
     return sts;
+#endif
 }
 
 int qat_aes_ccm_stream_update(void *vctx, unsigned char *out,
@@ -203,9 +270,6 @@ int qat_aes_ccm_stream_update(void *vctx, unsigned char *out,
                               const unsigned char *in, size_t inl)
 {
     QAT_PROV_CCM_CTX *ctx = (QAT_PROV_CCM_CTX *) vctx;
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    QAT_EVP_CIPHER sw_aes_ccm_cipher;
-# endif
 
     if (inl == 0) {
         *outl = 0;
@@ -216,19 +280,19 @@ int qat_aes_ccm_stream_update(void *vctx, unsigned char *out,
         QATerr(ERR_LIB_PROV, QAT_R_OUTPUT_BUFFER_TOO_SMALL);
         return 0;
     }
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    if (ctx->nid == NID_aes_192_ccm || ctx->nid == NID_aes_256_ccm) {
-        sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
-        if (sw_aes_ccm_cipher.cupdate == NULL)
-            return 0;
-
-        if (sw_aes_ccm_cipher.cupdate(ctx->sw_ctx, out, outl,
-                                             outsize, in, inl) <= 0) {
-            return 0;
-        }
-
-        return 1;
-    }
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
+#  ifdef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
 # endif
 
     if (qat_hw_aes_ccm_offload) {
@@ -237,9 +301,24 @@ int qat_aes_ccm_stream_update(void *vctx, unsigned char *out,
             return 0;
         }
     }
+    else {
+        /* Set *outl to NULL when offload is disabled to avoid garbage values and prevent errors. */
+        *outl = 0;
+    }
+    return 1;
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
+    QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
+    if (sw_aes_ccm_cipher.cupdate == NULL)
+        return 0;
+    if (sw_aes_ccm_cipher.cupdate(ctx->sw_ctx, out,
+                            outl, outsize, in, inl) <= 0)
+        return 0;
 
     return 1;
-
+#endif
 }
 
 int qat_aes_ccm_stream_final(void *vctx, unsigned char *out,
@@ -247,32 +326,45 @@ int qat_aes_ccm_stream_final(void *vctx, unsigned char *out,
 {
     int i = 0;
     QAT_PROV_CCM_CTX *ctx = (QAT_PROV_CCM_CTX *) vctx;
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    QAT_EVP_CIPHER sw_aes_ccm_cipher;
-# endif
 
     if (!qat_prov_is_running())
         return 0;
 
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    if (ctx->nid == NID_aes_192_ccm || ctx->nid == NID_aes_256_ccm) {
-        sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
-        if (sw_aes_ccm_cipher.cfinal == NULL)
-            return 0;
-        i = sw_aes_ccm_cipher.cfinal(ctx->sw_ctx, out, outl, outsize);
-        *outl = 0;
-        return 1;
-    }
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
+#  ifdef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
 # endif
 
-    if (qat_hw_aes_ccm_offload)
+    if (qat_hw_aes_ccm_offload) {
         i = qat_aes_ccm_cipher(ctx, out, outl, outsize, NULL, 0);
 
-    if (i <= 0)
-        return 0;
+        if (i <= 0)
+            return 0;
+    }
 
     *outl = 0;
     return 1;
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
+    QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
+    if (sw_aes_ccm_cipher.cfinal == NULL)
+        return 0;
+    i = sw_aes_ccm_cipher.cfinal(ctx->sw_ctx, out, outl, outsize);
+    *outl = 0;
+    return 1;
+#endif
 }
 
 int qat_aes_ccm_do_cipher(void *vctx, unsigned char *out,
@@ -280,9 +372,6 @@ int qat_aes_ccm_do_cipher(void *vctx, unsigned char *out,
                           const unsigned char *in, size_t inl)
 {
     QAT_PROV_CCM_CTX *ctx = (QAT_PROV_CCM_CTX *) vctx;
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    QAT_EVP_CIPHER sw_aes_ccm_cipher;
-# endif
 
     if (!qat_prov_is_running())
         return 0;
@@ -291,19 +380,19 @@ int qat_aes_ccm_do_cipher(void *vctx, unsigned char *out,
         QATerr(ERR_LIB_PROV, QAT_R_OUTPUT_BUFFER_TOO_SMALL);
         return 0;
     }
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    if (ctx->nid == NID_aes_192_ccm || ctx->nid == NID_aes_256_ccm) {
-        sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
-        if (sw_aes_ccm_cipher.cupdate == NULL)
-            return 0;
-
-        if (sw_aes_ccm_cipher.cupdate(ctx->sw_ctx, out, outl,
-                                             outsize, in, inl) <= 0) {
-            return 0;
-        }
-
-        return 1;
-    }
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
+#  ifdef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
 # endif
     if (qat_hw_aes_ccm_offload) {
         if (qat_aes_ccm_cipher(ctx, out, outl, outsize, in, inl) <= 0) {
@@ -313,6 +402,18 @@ int qat_aes_ccm_do_cipher(void *vctx, unsigned char *out,
 
     *outl = inl;
     return 1;
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
+    QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
+    if (sw_aes_ccm_cipher.cupdate == NULL)
+        return 0;
+    if (sw_aes_ccm_cipher.cupdate(ctx->sw_ctx, out, outl,
+                                          outsize, in, inl) <= 0)
+        return 0;
+    return 1;
+#endif
 }
 
 int qat_aes_ccm_get_ctx_params(void *vctx, OSSL_PARAM params[])
@@ -320,14 +421,19 @@ int qat_aes_ccm_get_ctx_params(void *vctx, OSSL_PARAM params[])
     QAT_PROV_CCM_CTX *ctx = (QAT_PROV_CCM_CTX *) vctx;
     OSSL_PARAM *p;
 
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    if (ctx->nid == NID_aes_192_ccm || ctx->nid == NID_aes_256_ccm) {
-        if (ctx->sw_ctx) {
-            QAT_EVP_CIPHER sw_aes_ccm_cipher =
-                get_default_cipher_aes_ccm(ctx->nid);
-            return sw_aes_ccm_cipher.get_ctx_params(ctx->sw_ctx, params);
-        }
-    }
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
+#  ifdef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
 # endif
 
     p = OSSL_PARAM_locate(params, OSSL_CIPHER_PARAM_IVLEN);
@@ -382,42 +488,40 @@ int qat_aes_ccm_get_ctx_params(void *vctx, OSSL_PARAM params[])
     }
     p = OSSL_PARAM_locate(params, OSSL_CIPHER_PARAM_AEAD_TAG);
     if (p != NULL) {
-# ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
-        if (ctx->packet_size > qat_pkt_threshold_table_get_threshold(ctx->nid)) {
-            if (!ctx->enc || !ctx->tag_set) {
-                QATerr(ERR_LIB_PROV, QAT_R_INVALID_TAG);
-                return 0;
-            }
-        }
-# else
-        if (!ctx->enc || !ctx->tag_set) {
+        if (!ctx->enc) {
             QATerr(ERR_LIB_PROV, QAT_R_INVALID_TAG);
             return 0;
         }
-# endif
         if (p->data_type != OSSL_PARAM_OCTET_STRING) {
             QATerr(ERR_LIB_PROV, QAT_R_FAILED_TO_SET_PARAMETER);
             return 0;
         }
-        if (qat_hw_aes_ccm_offload)
-            qat_aes_ccm_ctrl(ctx, EVP_CTRL_AEAD_GET_TAG, p->data_size, p->data);
 
-        ctx->tag_set = 0;
+        memcpy(p->data, ctx->buf, p->data_size);
+
         ctx->iv_set = 0;
         ctx->len_set = 0;
-# ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
-        if (ctx->packet_size > qat_pkt_threshold_table_get_threshold(ctx->nid))
+        if (ctx->tag_set) {
+            ctx->tag_set = 0;
             return 1;
-# endif
+        }
 
     }
-# ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
+
     if (ctx->sw_ctx) {
         QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
         sw_aes_ccm_cipher.get_ctx_params(ctx->sw_ctx, params);
     }
-# endif
     return 1;
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
+    QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
+    if (ctx->sw_ctx || (ctx->sw_ctx = sw_aes_ccm_cipher.newctx(ctx)))
+        return sw_aes_ccm_cipher.get_ctx_params(ctx->sw_ctx, params);
+    return 0;
+#endif
 }
 
 int qat_aes_ccm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
@@ -429,14 +533,19 @@ int qat_aes_ccm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
     if (params == NULL)
         return 1;
 
-# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE)
-    if (ctx->nid == NID_aes_192_ccm || ctx->nid == NID_aes_256_ccm) {
-        if (ctx->sw_ctx) {
-            QAT_EVP_CIPHER sw_aes_ccm_cipher =
-                get_default_cipher_aes_ccm(ctx->nid);
-            return sw_aes_ccm_cipher.set_ctx_params(ctx->sw_ctx, params);
-        }
-    }
+# if !defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE)
+#  ifdef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_128_ccm)
+        goto end;
+#  else
+    goto end;
+#  endif
+# else
+#  ifndef QAT_INSECURE_ALGO
+    if (ctx->nid != NID_aes_256_ccm)
+        goto end;
+#  endif
 # endif
     p = OSSL_PARAM_locate_const(params, OSSL_CIPHER_PARAM_AEAD_TAG);
     if (p != NULL) {
@@ -457,8 +566,6 @@ int qat_aes_ccm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
             memcpy(ctx->buf, p->data, p->data_size);
             ctx->tag_set = 1;
         }
-        if (qat_hw_aes_ccm_offload)
-            qat_aes_ccm_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, p->data_size, p->data);
 
         ctx->M = p->data_size;
     }
@@ -491,6 +598,8 @@ int qat_aes_ccm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
         if (qat_hw_aes_ccm_offload)
             sz = qat_aes_ccm_ctrl(ctx, EVP_CTRL_AEAD_TLS1_AAD, p->data_size,
                                   p->data);
+	else
+	    sz = qat_aes_ccm_tls_init(ctx, p->data, p->data_size);
 
         if (sz == 0) {
             QATerr(ERR_LIB_PROV, QAT_R_INVALID_DATA);
@@ -513,13 +622,23 @@ int qat_aes_ccm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
             }
         }
     }
-# ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
+
     if (ctx->sw_ctx) {
         QAT_EVP_CIPHER sw_aes_ccm_cipher = get_default_cipher_aes_ccm(ctx->nid);
         sw_aes_ccm_cipher.set_ctx_params(ctx->sw_ctx, params);
     }
-# endif
     return 1;
+# if (!defined(QAT20_OOT) && !defined(QAT_HW_INTREE) \
+     && !defined(QAT_HW_FBSD_OOT) && !defined(QAT_HW_FBSD_INTREE) \
+     && defined(QAT_INSECURE_ALGO)) || !defined(QAT_INSECURE_ALGO)
+end:
+    if (ctx->sw_ctx) {
+        QAT_EVP_CIPHER sw_aes_ccm_cipher =
+              get_default_cipher_aes_ccm(ctx->nid);
+        return sw_aes_ccm_cipher.set_ctx_params(ctx->sw_ctx, params);
+    }
+    return 0;
+#endif
 }
 
 int qat_aes_ccm_generic_get_params(OSSL_PARAM params[], unsigned int md,
