@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -61,6 +61,42 @@ static OSSL_FUNC_cipher_settable_ctx_params_fn qat_aes_settable_ctx_params;
 
 # define ossl_assert(x) ((x) != 0)
 
+const char *qat_get_cipher_name_from_nid(int nid)
+{
+    switch (nid) {
+        case NID_aes_128_cbc_hmac_sha1:
+            return "AES-128-CBC-HMAC-SHA1";
+        case NID_aes_256_cbc_hmac_sha1:
+            return "AES-256-CBC-HMAC-SHA1";
+        case NID_aes_128_cbc_hmac_sha256:
+            return "AES-128-CBC-HMAC-SHA256";
+        case NID_aes_256_cbc_hmac_sha256:
+            return "AES-256-CBC-HMAC-SHA256";
+        default:
+            WARN("Invalid nid %d\n", nid);
+            return NULL;
+    }
+}
+
+PROV_EVP_CIPHER get_default_cipher_aes_cbc(int nid)
+{
+    static PROV_EVP_CIPHER cbc_cipher;
+    static int initialized = 0;
+    if (!initialized) {
+        PROV_EVP_CIPHER *cipher =
+            (PROV_EVP_CIPHER *) EVP_CIPHER_fetch(NULL, qat_get_cipher_name_from_nid(nid),
+                                                "provider=default");
+        if (cipher) {
+            cbc_cipher = *cipher;
+            EVP_CIPHER_free((EVP_CIPHER *)cipher);
+            initialized = 1;
+        } else {
+            WARN("EVP_CIPHER_fetch from default provider failed");
+        }
+    }
+    return cbc_cipher;
+}
+
 static int qat_cipher_generic_initiv(PROV_CIPHER_CTX *ctx, const unsigned char *iv,
                                size_t ivlen)
 {
@@ -87,7 +123,10 @@ static int qat_aes_einit(void *vctx, const unsigned char *key, size_t keylen,
 
     if (!qat_prov_is_running())
         return 0;
-
+# ifndef QAT_INSECURE_ALGO
+    if (ctx->nid == NID_aes_128_cbc_hmac_sha256)
+        goto end;
+# endif
     if (key != NULL && !qat_chained_ciphers_init(ctx, key, keylen, iv, ivlen, 1)){
         WARN("qat_chained_ciphers_init failed\n");
         return 0;
@@ -116,6 +155,14 @@ static int qat_aes_einit(void *vctx, const unsigned char *key, size_t keylen,
     }
 
     return qat_aes_set_ctx_params(ctx, params);
+#ifndef QAT_INSECURE_ALGO
+end:
+    const OSSL_PARAM ps[2] = { OSSL_PARAM_END, OSSL_PARAM_END };
+    PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->nid);
+    if (ctx->sw_ctx || (ctx->sw_ctx = sw_aes_cbc_cipher.newctx(ctx)))
+        return sw_aes_cbc_cipher.einit(ctx->sw_ctx, key, keylen, iv, ivlen, ps);
+    return 0;
+#endif
 }
 
 static int qat_aes_dinit(void *vctx, const unsigned char *key, size_t keylen,
@@ -127,7 +174,10 @@ static int qat_aes_dinit(void *vctx, const unsigned char *key, size_t keylen,
     ctx->bufsz = 0;
     ctx->updated = 0;
     ctx->enc = 0;
-
+# ifndef QAT_INSECURE_ALGO
+    if (ctx->nid == NID_aes_128_cbc_hmac_sha256)
+        goto end;
+# endif
     if (!qat_prov_is_running())
         return 0;
 
@@ -159,6 +209,14 @@ static int qat_aes_dinit(void *vctx, const unsigned char *key, size_t keylen,
     }
 
     return qat_aes_set_ctx_params(ctx, params);
+#ifndef QAT_INSECURE_ALGO
+end:
+    const OSSL_PARAM ps[2] = { OSSL_PARAM_END, OSSL_PARAM_END };
+    PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->nid);
+    if (ctx->sw_ctx || (ctx->sw_ctx = sw_aes_cbc_cipher.newctx(ctx)))
+        return sw_aes_cbc_cipher.dinit(ctx->sw_ctx, key, keylen, iv, ivlen, ps);
+    return 0;
+#endif
 }
 
 static const OSSL_PARAM cipher_aes_known_settable_ctx_params[] = {
@@ -188,10 +246,12 @@ static int qat_aes_set_ctx_params(void *vctx, const OSSL_PARAM params[])
 # if !defined(OPENSSL_NO_MULTIBLOCK)
     EVP_CTRL_TLS1_1_MULTIBLOCK_PARAM mb_param;
 # endif
-
     if (params == NULL)
         return 1;
-
+# ifndef QAT_INSECURE_ALGO
+    if (ctx->base.nid == NID_aes_128_cbc_hmac_sha256)
+        goto end;
+# endif
     p = OSSL_PARAM_locate_const(params, OSSL_CIPHER_PARAM_AEAD_MAC_KEY);
     if (p != NULL) {
         if (p->data_type != OSSL_PARAM_OCTET_STRING) {
@@ -314,14 +374,30 @@ static int qat_aes_set_ctx_params(void *vctx, const OSSL_PARAM params[])
             ctx->base.removetlsfixed -= AES_BLOCK_SIZE;
         }
     }
+
+    if (ctx->base.sw_ctx) {
+        PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->base.nid);
+        sw_aes_cbc_cipher.set_ctx_params(ctx->base.sw_ctx, params);
+    }
     return ret;
+#ifndef QAT_INSECURE_ALGO
+end:
+    if (ctx->base.sw_ctx) {
+        PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->base.nid);
+        return sw_aes_cbc_cipher.set_ctx_params(ctx->base.sw_ctx, params);
+    }
+    return 0;
+#endif
 }
 
 static int qat_aes_get_ctx_params(void *vctx, OSSL_PARAM params[])
 {
     PROV_AES_HMAC_SHA_CTX *ctx = (PROV_AES_HMAC_SHA_CTX *)vctx;
     OSSL_PARAM *p;
-
+# ifndef QAT_INSECURE_ALGO
+    if (ctx->base.nid == NID_aes_128_cbc_hmac_sha256)
+        goto end;
+# endif
 # if !defined(OPENSSL_NO_MULTIBLOCK)
     p = OSSL_PARAM_locate(params, OSSL_CIPHER_PARAM_TLS1_MULTIBLOCK_MAX_BUFSIZE);
     if (p != NULL) {
@@ -378,6 +454,15 @@ static int qat_aes_get_ctx_params(void *vctx, OSSL_PARAM params[])
         return 0;
     }
     return 1;
+#ifndef QAT_INSECURE_ALGO
+end:
+    PROV_EVP_CIPHER sw_aes_cbc_cipher =
+            get_default_cipher_aes_cbc(ctx->base.nid);
+    ctx->base.sw_ctx = sw_aes_cbc_cipher.newctx(ctx);
+    if (ctx->base.sw_ctx)
+        return sw_aes_cbc_cipher.get_ctx_params(ctx->base.sw_ctx, params);
+    return 0;
+#endif
 }
 
 static const OSSL_PARAM cipher_aes_known_gettable_ctx_params[] = {
@@ -406,6 +491,7 @@ static void *qat_aes_cbc_hmac_sha1_newctx(void *provctx, size_t kbits,
 {
     PROV_AES_HMAC_SHA1_CTX *ctx = NULL;
     PROV_CIPHER_CTX *base_ctx = NULL;
+    PROV_EVP_CIPHER sw_aes_cbc_cipher;
 
     if (!qat_prov_is_running())
         return NULL;
@@ -420,15 +506,16 @@ static void *qat_aes_cbc_hmac_sha1_newctx(void *provctx, size_t kbits,
             WARN("qat_cipher_ctx zalloc failed.\n");
             goto err;
         }
+	sw_aes_cbc_cipher = get_default_cipher_aes_cbc(base_ctx->nid);
 #ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
-        base_ctx->sw_ctx = EVP_CIPHER_CTX_new();
+	base_ctx->sw_ctx = sw_aes_cbc_cipher.newctx(base_ctx);
         if (base_ctx->sw_ctx == NULL){
             WARN("base_ctx->sw_ctx zalloc failed.\n");
             goto err;
         }
 #else
         if (enable_sw_fallback){
-            base_ctx->sw_ctx = EVP_CIPHER_CTX_new();
+	    base_ctx->sw_ctx = sw_aes_cbc_cipher.newctx(base_ctx);
             if (base_ctx->sw_ctx == NULL){
                 WARN("base_ctx->sw_ctx zalloc failed.\n");
                 goto err;
@@ -466,6 +553,7 @@ static void *qat_aes_cbc_hmac_sha256_newctx(void *provctx, size_t kbits,
 {
     PROV_AES_HMAC_SHA256_CTX *ctx = NULL;
     PROV_CIPHER_CTX *base_ctx = NULL;
+    PROV_EVP_CIPHER sw_aes_cbc_cipher;
 
     if (!qat_prov_is_running())
         return NULL;
@@ -480,8 +568,9 @@ static void *qat_aes_cbc_hmac_sha256_newctx(void *provctx, size_t kbits,
             WARN("qat_cipher_ctx zalloc failed.\n");
             goto err;
         }
+	sw_aes_cbc_cipher = get_default_cipher_aes_cbc(base_ctx->nid);
 #ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
-        base_ctx->sw_ctx = EVP_CIPHER_CTX_new();
+	base_ctx->sw_ctx = sw_aes_cbc_cipher.newctx(base_ctx);
         if (base_ctx->sw_ctx == NULL){
             WARN("base_ctx->sw_ctx zalloc failed.\n");
             goto err;
@@ -489,7 +578,7 @@ static void *qat_aes_cbc_hmac_sha256_newctx(void *provctx, size_t kbits,
         EVP_CIPHER_CTX_init(base_ctx->sw_ctx);
 #else
         if (enable_sw_fallback){
-            base_ctx->sw_ctx = EVP_CIPHER_CTX_new();
+	    base_ctx->sw_ctx = sw_aes_cbc_cipher.newctx(base_ctx);
             if (base_ctx->sw_ctx == NULL){
                 WARN("base_ctx->sw_ctx zalloc failed.\n");
                 goto err;
@@ -571,7 +660,10 @@ int qat_aes_cbc_cipher_do_cipher(void *vctx, unsigned char *out, size_t *outl,
                                size_t inl)
 {
     PROV_CIPHER_CTX *ctx = (PROV_CIPHER_CTX *)vctx;
-
+# ifndef QAT_INSECURE_ALGO
+    if (ctx->nid == NID_aes_128_cbc_hmac_sha256)
+        goto end;
+# endif
     if (!qat_prov_is_running())
         return 0;
 
@@ -580,18 +672,36 @@ int qat_aes_cbc_cipher_do_cipher(void *vctx, unsigned char *out, size_t *outl,
         return 0;
     }
 
-    if (qat_chained_ciphers_do_cipher(ctx, out, in, inl) <= 0) {
+    if (qat_chained_ciphers_do_cipher(ctx, out, outl, outsize, in, inl) <= 0) {
         QATerr(ERR_LIB_PROV, PROV_R_CIPHER_OPERATION_FAILED);
         return 0;
     }
 
     *outl = inl;
     return 1;
+#ifndef QAT_INSECURE_ALGO
+end:
+    if (ctx->sw_ctx) {
+        PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->nid);
+        return sw_aes_cbc_cipher.cupdate(ctx->sw_ctx, out, outl, outsize, in, inl);
+    }
+    return 0;
+#endif
 }
 
 int qat_cipher_generic_stream_final(void *vctx, unsigned char *out,
                                      size_t *outl, size_t outsize)
 {
+# ifndef QAT_INSECURE_ALGO
+    PROV_CIPHER_CTX *ctx = (PROV_CIPHER_CTX *)vctx;
+    if (ctx->nid == NID_aes_128_cbc_hmac_sha256) {
+	if (ctx->sw_ctx) {
+             PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->nid);
+             return sw_aes_cbc_cipher.cfinal(ctx->sw_ctx, out, outl, outsize);
+	}
+    return 0;
+    }
+# endif
     if (!qat_prov_is_running())
         return 0;
 
@@ -605,6 +715,10 @@ int qat_cipher_generic_stream_update(void *vctx, unsigned char *out,
 {
     PROV_CIPHER_CTX *ctx = (PROV_CIPHER_CTX *)vctx;
 
+# ifndef QAT_INSECURE_ALGO
+    if (ctx->nid == NID_aes_128_cbc_hmac_sha256)
+        goto end;
+# endif
     if (inl == 0) {
         *outl = 0;
         return 1;
@@ -615,7 +729,7 @@ int qat_cipher_generic_stream_update(void *vctx, unsigned char *out,
         return 0;
     }
 
-    if (qat_chained_ciphers_do_cipher(ctx, out, in, inl) <= 0){
+    if (qat_chained_ciphers_do_cipher(ctx, out, outl, outsize, in, inl) <= 0){
         QATerr(ERR_LIB_PROV, PROV_R_CIPHER_OPERATION_FAILED);
         return 0;
     }
@@ -655,6 +769,14 @@ int qat_cipher_generic_stream_update(void *vctx, unsigned char *out,
     }
 
     return 1;
+#ifndef QAT_INSECURE_ALGO
+end:
+    if (ctx->sw_ctx) {
+        PROV_EVP_CIPHER sw_aes_cbc_cipher = get_default_cipher_aes_cbc(ctx->nid);
+        return sw_aes_cbc_cipher.cupdate(ctx->sw_ctx, out, outl, outsize, in, inl);
+    }
+    return 0;
+#endif
 }
 
 static const OSSL_PARAM cipher_known_gettable_params[] = {
