@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@
 #ifdef ENABLE_QAT_SW_GCM
 #include "qat_sw_gcm.h"
 #endif
-
+int nid;
 #define AES_GCM_IV_MIN_SIZE     (64 / 8)
 
 #define UNINITIALISED_SIZET ((int)-1)
@@ -63,6 +63,44 @@
 #define AEAD_FLAGS (PROV_CIPHER_FLAG_AEAD | PROV_CIPHER_FLAG_CUSTOM_IV)
 
 #if defined(ENABLE_QAT_HW_GCM) || defined(ENABLE_QAT_SW_GCM)
+const char *qat_gcm_cipher_name(int nid)
+{
+    switch (nid) {
+    case NID_aes_128_gcm:
+        return LN_aes_128_gcm;
+    case NID_aes_192_gcm:
+        return LN_aes_192_gcm;
+    case NID_aes_256_gcm:
+        return LN_aes_256_gcm;
+    default:
+        WARN("Invalid nid %d\n", nid);
+        return NULL;
+    }
+}
+QAT_EVP_CIPHER get_default_cipher_aes_gcm(int nid)
+{
+    static QAT_EVP_CIPHER gcm_cipher;
+    static int initialized = 0;
+    if (!initialized) {
+        QAT_EVP_CIPHER *cipher =
+            (QAT_EVP_CIPHER *) EVP_CIPHER_fetch(NULL, qat_gcm_cipher_name(nid),
+                                                "provider=default");
+        if (cipher) {
+            gcm_cipher = *cipher;
+            EVP_CIPHER_free((EVP_CIPHER *)cipher);
+            initialized = 1;
+        } else {
+            WARN("EVP_CIPHER_fetch from default provider failed");
+        }
+    }
+    return gcm_cipher;
+}
+
+int qat_aes_gcm_ctx_get_nid(const QAT_AES_GCM_CTX *ctx)
+{
+    return ctx->cipher->nid;
+}
+
 void qat_gcm_initctx(void *provctx, QAT_GCM_CTX *ctx, size_t keybits,
                      size_t ivlen_min)
 {
@@ -101,7 +139,15 @@ int qat_gcm_get_ctx_params(void *vctx, OSSL_PARAM params[])
     QAT_GCM_CTX *ctx = (QAT_GCM_CTX *)vctx;
     OSSL_PARAM *p;
     size_t sz;
-
+    int ret = 1;
+#if defined(ENABLE_QAT_HW_GCM) || (defined(ENABLE_QAT_SW_GCM) \
+                && !defined(QAT_INSECURE_ALGO))
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+# ifndef QAT_INSECURE_ALGO
+    if (nid == NID_aes_128_gcm)
+        goto fallback;
+# endif
+#endif
     p = OSSL_PARAM_locate(params, OSSL_CIPHER_PARAM_IVLEN);
     if (p != NULL && !OSSL_PARAM_set_size_t(p, ctx->iv_len)) {
         QATerr(ERR_LIB_PROV, PROV_R_FAILED_TO_SET_PARAMETER);
@@ -163,15 +209,46 @@ int qat_gcm_get_ctx_params(void *vctx, OSSL_PARAM params[])
             || sz > EVP_GCM_TLS_TAG_LEN
             || !ctx->enc
             || ctx->tag_len == UNINITIALISED_SIZET) {
-            QATerr(ERR_LIB_PROV, PROV_R_INVALID_TAG);
+#ifdef ENABLE_QAT_HW_GCM
+            if (ctx->tag_set)
+                QATerr(ERR_LIB_PROV, PROV_R_INVALID_TAG);
+            ret = 0;
+            ctx->tag_set = 0;
+	    goto end;
+#else
             return 0;
+#endif
         }
         if (!OSSL_PARAM_set_octet_string(p, ctx->buf, sz)) {
             QATerr(ERR_LIB_PROV, PROV_R_FAILED_TO_SET_PARAMETER);
             return 0;
         }
+#ifdef ENABLE_QAT_HW_GCM
+        if (!ctx->tag_set) {
+            ctx->tag_set = 0;
+            goto end;
+        }
+        ctx->tag_set = 0;
+#endif
     }
+
     return 1;
+
+#ifdef ENABLE_QAT_HW_GCM
+end:
+    if (ctx->sw_ctx) {
+        QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+        ret = sw_aes_gcm_cipher.get_ctx_params(ctx->sw_ctx, params);
+    }
+#endif
+    return ret;
+#ifndef QAT_INSECURE_ALGO
+fallback:
+    QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+    if (ctx->sw_ctx || (ctx->sw_ctx = sw_aes_gcm_cipher.newctx(ctx)))
+        return sw_aes_gcm_cipher.get_ctx_params(ctx->sw_ctx, params);
+    return 0;
+#endif
 }
 
 int qat_gcm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
@@ -180,7 +257,14 @@ int qat_gcm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
     const OSSL_PARAM *p;
     size_t sz = 0;
     void *vp;
-
+#if defined(ENABLE_QAT_HW_GCM) || (defined(ENABLE_QAT_SW_GCM) \
+                && !defined(QAT_INSECURE_ALGO))
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+# ifndef QAT_INSECURE_ALGO
+    if (nid == NID_aes_128_gcm)
+        goto fallback;
+# endif
+#endif
     if (params == NULL)
         return 1;
 
@@ -196,6 +280,9 @@ int qat_gcm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
             return 0;
         }
         ctx->tag_len = sz;
+#ifdef ENABLE_QAT_HW_GCM
+        ctx->tag_set = 1;
+#endif
     }
 
     p = OSSL_PARAM_locate_const(params, OSSL_CIPHER_PARAM_AEAD_IVLEN);
@@ -260,14 +347,34 @@ int qat_gcm_set_ctx_params(void *vctx, const OSSL_PARAM params[])
             return 0;
     }
 
+#ifdef ENABLE_QAT_HW_GCM
+    if (ctx->sw_ctx) {
+        QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+        sw_aes_gcm_cipher.set_ctx_params(ctx->sw_ctx, params);
+    }
+#endif
     return 1;
+#ifndef QAT_INSECURE_ALGO
+fallback:
+    if (ctx->sw_ctx) {
+        QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+        return sw_aes_gcm_cipher.set_ctx_params(ctx->sw_ctx, params);
+    }
+    return 0;
+#endif
 }
 
-int qat_gcm_einit(void *ctx, const unsigned char* inkey,
+int qat_gcm_einit(void *vctx, const unsigned char* inkey,
                   int keylen, const unsigned char* iv, int ivlen,
                   int enc)
 {
     int sts = 0;
+    QAT_GCM_CTX *ctx = (QAT_GCM_CTX *)vctx;
+#ifndef QAT_INSECURE_ALGO
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+    if (nid == NID_aes_128_gcm)
+        goto fallback;
+#endif
 #ifdef ENABLE_QAT_HW_GCM
     if (qat_hw_gcm_offload)
         sts = qat_aes_gcm_init(ctx, inkey, keylen, iv, ivlen, 1);
@@ -277,13 +384,27 @@ int qat_gcm_einit(void *ctx, const unsigned char* inkey,
         sts = vaesgcm_ciphers_init(ctx, inkey, iv, 1);
 #endif
     return sts;
+# ifndef QAT_INSECURE_ALGO
+fallback:
+    OSSL_PARAM params[2] = { OSSL_PARAM_END, OSSL_PARAM_END };
+    QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+    if (ctx->sw_ctx || (ctx->sw_ctx = sw_aes_gcm_cipher.newctx(ctx)))
+        return sw_aes_gcm_cipher.einit(ctx->sw_ctx, inkey, keylen, iv, ivlen, params);
+    return 0;
+# endif
 }
 
-int qat_gcm_dinit(void *ctx, const unsigned char* inkey,
+int qat_gcm_dinit(void *vctx, const unsigned char* inkey,
                   int keylen, const unsigned char* iv, int ivlen,
                   int enc)
 {
     int sts = 0;
+    QAT_GCM_CTX *ctx = (QAT_GCM_CTX *)vctx;
+#ifndef QAT_INSECURE_ALGO
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+    if (nid == NID_aes_128_gcm)
+        goto fallback;
+#endif
 #ifdef ENABLE_QAT_HW_GCM
     if (qat_hw_gcm_offload)
         sts = qat_aes_gcm_init(ctx, inkey, keylen, iv, ivlen, 0);
@@ -293,6 +414,16 @@ int qat_gcm_dinit(void *ctx, const unsigned char* inkey,
         sts = vaesgcm_ciphers_init(ctx, inkey, iv, 0);
 #endif
    return sts;
+#ifndef QAT_INSECURE_ALGO
+fallback:
+    OSSL_PARAM params[2] = { OSSL_PARAM_END, OSSL_PARAM_END };
+    QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+    if (!ctx->sw_ctx)
+        ctx->sw_ctx = sw_aes_gcm_cipher.newctx(ctx);
+    unsigned int pad = 0;
+    params[0] = OSSL_PARAM_construct_uint(OSSL_CIPHER_PARAM_PADDING, &pad);
+    return sw_aes_gcm_cipher.dinit(ctx->sw_ctx, inkey, keylen, iv, ivlen, params);
+#endif
 }
 
 
@@ -308,7 +439,12 @@ int qat_gcm_stream_update(void *vctx, unsigned char *out,
     int ret = 0;
     QAT_GCM_CTX *ctx = (QAT_GCM_CTX *)vctx;
 #endif
-
+#ifndef QAT_INSECURE_ALGO
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+    if (nid == NID_aes_128_gcm) {
+        goto fallback;
+    }
+#endif
 #ifndef ENABLE_QAT_FIPS
     if (inl == 0) {
         *outl = 0;
@@ -323,7 +459,7 @@ int qat_gcm_stream_update(void *vctx, unsigned char *out,
     }
 #ifdef ENABLE_QAT_HW_GCM
     if (qat_hw_gcm_offload) {
-        if ((ret = qat_aes_gcm_cipher(ctx, out, outl, in, inl)) <= 0) {
+        if ((ret = qat_aes_gcm_cipher(ctx, out, outl, outsize, in, inl)) <= 0) {
             QATerr(ERR_LIB_PROV, PROV_R_CIPHER_OPERATION_FAILED);
             goto end;
         }
@@ -345,6 +481,16 @@ end:
     qat_fips_service_indicator = 0;
 #endif
     return ret;
+#ifndef QAT_INSECURE_ALGO
+fallback:
+    QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+    if (sw_aes_gcm_cipher.cupdate == NULL)
+        return 0;
+    if (sw_aes_gcm_cipher.cupdate(ctx->sw_ctx, out, outl, outsize, in, inl) <= 0) {
+        return 0;
+    }
+    return 1;
+#endif
 }
 
 int qat_gcm_stream_final(void *vctx, unsigned char *out,
@@ -357,6 +503,11 @@ int ret = 0;
 
 #if defined(ENABLE_QAT_HW_GCM) || defined(ENABLE_QAT_SW_GCM)
     QAT_GCM_CTX *ctx = (QAT_GCM_CTX *)vctx;
+#endif
+#ifndef QAT_INSECURE_ALGO
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+    if (nid == NID_aes_128_gcm)
+        goto fallback;
 #endif
     int i = 0;
 
@@ -364,7 +515,7 @@ int ret = 0;
         goto end;
 #ifdef ENABLE_QAT_HW_GCM
     if (qat_hw_gcm_offload)
-        i = qat_aes_gcm_cipher(ctx, out, outl, NULL, 0);
+        i = qat_aes_gcm_cipher(ctx, out, outl, outsize, NULL, 0);
 #endif
 
 #ifdef ENABLE_QAT_SW_GCM
@@ -383,6 +534,14 @@ end:
     qat_fips_service_indicator = 0;
 #endif
     return ret;
+#ifndef QAT_INSECURE_ALGO
+fallback:
+    QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+    if (sw_aes_gcm_cipher.cfinal == NULL)
+        return 0;
+    i = sw_aes_gcm_cipher.cfinal(ctx->sw_ctx, out, outl, outsize);
+    return i;
+#endif
 }
 
 int qat_gcm_cipher(void *vctx, unsigned char *out,
@@ -393,6 +552,11 @@ int qat_gcm_cipher(void *vctx, unsigned char *out,
 #if defined(ENABLE_QAT_HW_GCM) || defined(ENABLE_QAT_SW_GCM)
     QAT_GCM_CTX *ctx = (QAT_GCM_CTX *)vctx;
 #endif
+#ifndef QAT_INSECURE_ALGO
+    nid = qat_aes_gcm_ctx_get_nid((QAT_AES_GCM_CTX *)ctx);
+    if (nid == NID_aes_128_gcm)
+        goto fallback;
+#endif
 #ifdef ENABLE_QAT_FIPS
     qat_fips_service_indicator = 1;
 #endif
@@ -406,7 +570,7 @@ int qat_gcm_cipher(void *vctx, unsigned char *out,
     }
 #ifdef ENABLE_QAT_HW_GCM
     if (qat_hw_gcm_offload) {
-        if (qat_aes_gcm_cipher(ctx, out, outl, in, inl) <= 0)
+        if (qat_aes_gcm_cipher(ctx, out, outl, outsize, in, inl) <= 0)
             goto end;
     }
 #endif
@@ -425,6 +589,15 @@ end:
     qat_fips_service_indicator = 0;
 #endif
     return ret;
+#ifndef QAT_INSECURE_ALGO
+fallback:
+    QAT_EVP_CIPHER sw_aes_gcm_cipher = get_default_cipher_aes_gcm(nid);
+    if (sw_aes_gcm_cipher.cupdate == NULL)
+        return 0;
+    if (sw_aes_gcm_cipher.cupdate(ctx->sw_ctx, out, outl, outsize, in, inl) <= 0)
+        return 0;
+    return 1;
+#endif
 }
 
 static void qat_aes_gcm_freectx(void *vctx)
