@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -63,8 +63,9 @@
 #include <openssl/err.h>
 #include <openssl/proverr.h>
 #include <openssl/params.h>
+#include <openssl/evp.h>
+#include "e_qat.h"
 
-typedef int CRYPTO_REFERENCE_COUNT;
 #define FFC_UNVERIFIABLE_GINDEX -1
 #define FFC_PARAM_FLAG_VALIDATE_PQ 0x01
 #define FFC_PARAM_FLAG_VALIDATE_G 0x02
@@ -123,14 +124,16 @@ struct dh_st
     BIGNUM *priv_key; /* x */
     int flags;
     BN_MONT_CTX *method_mont_p;
-    CRYPTO_REFERENCE_COUNT references;
+    QAT_CRYPTO_REF_COUNT references;
 #ifndef FIPS_MODULE
     CRYPTO_EX_DATA ex_data;
     ENGINE *engine;
 #endif
     OSSL_LIB_CTX *libctx;
     const DH_METHOD *meth;
+# if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_RWLOCK *lock;
+#endif
 
     /* Provider data */
     size_t dirty_cnt; /* If any key material changes, increment this */
