@@ -74,6 +74,7 @@
 #include "crypto_mb/ec_nistp256.h"
 #include "crypto_mb/ec_nistp384.h"
 #include "crypto_mb/ec_sm2.h"
+#include "crypto_mb/cpu_features.h"
 
 # if defined(ENABLE_QAT_FIPS) && defined(ENABLE_QAT_SW_ECDSA)
 extern int qat_fips_kat_test;
@@ -113,14 +114,17 @@ static inline int mb_ec_check_curve(int curve_type)
 
     switch (curve_type) {
     case NID_X9_62_prime256v1:
-        ret = EC_P256;
+        if (mbx_get_algo_info(MBX_ALGO_ECDSA_NIST_P256))
+            ret = EC_P256;
         break;
     case NID_secp384r1:
-        ret = EC_P384;
+        if (mbx_get_algo_info(MBX_ALGO_ECDSA_NIST_P384))
+            ret = EC_P384;
         break;
 #ifndef QAT_BORINGSSL
     case NID_sm2:
-        ret = EC_SM2;
+        if (mbx_get_algo_info(MBX_ALGO_EC_SM2))
+            ret = EC_SM2;
         break;
 #endif /* QAT_BORINGSSL */
     default:
