@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2020-2024 Intel Corporation.
+ *   Copyright(c) 2020-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,9 @@
 #ifndef QAT_SW_ECX_H
 # define QAT_SW_ECX_H
 
-# include <openssl/engine.h>
+# ifndef QAT_OPENSSL_PROVIDER
+#  include <openssl/engine.h>
+# endif
 # include <openssl/ossl_typ.h>
 
 
