@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023 Intel Corporation.
+ *   Copyright(c) 2023-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -86,7 +86,9 @@ static const unsigned char iv[] = {
 static int run_aesccm128_tls(void *args)
 {
     TEST_PARAMS *temp_args = (TEST_PARAMS *) args;
+#ifndef QAT_OPENSSL_PROVIDER
     ENGINE *e = temp_args->e;
+#endif
     int size = INPUT_TEXT_SIZE;
     int print_output = temp_args->print_output;
     int verify = temp_args->verify;
@@ -127,6 +129,7 @@ static int run_aesccm128_tls(void *args)
     EVP_CIPHER_CTX *ctx = NULL;
     EVP_CIPHER_CTX *dec_ctx = NULL;
 
+#ifndef QAT_OPENSSL_PROVIDER
     if (e != NULL) {
         const EVP_CIPHER *c = ENGINE_get_cipher(e, NID_aes_128_ccm);
 
@@ -135,6 +138,7 @@ static int run_aesccm128_tls(void *args)
             e = NULL;
         }
     }
+#endif
 
     if (input == NULL) {
         INFO("# FAIL: [%s] --- Initial parameters malloc failed ! \n",
@@ -165,7 +169,11 @@ static int run_aesccm128_tls(void *args)
      * IV used in the initialization of the cipher: in TLS case it is set
      * using ctrl() instead of EncryptInit_ex().
      */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit_ex(ctx, EVP_aes_128_ccm(), NULL, NULL, NULL);
+#else
     ret = EVP_EncryptInit_ex(ctx, EVP_aes_128_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed: ret %d\n",
              __func__, ret);
@@ -190,7 +198,11 @@ static int run_aesccm128_tls(void *args)
     }
 
     /* set 16-byte of key to the aes-ccm context. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit_ex(ctx, NULL, NULL, key, NULL);
+#else
     ret = EVP_EncryptInit_ex(ctx, EVP_aes_128_ccm(), e, key, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed for setting key: ret %d\n", __func__, ret);
         goto err;
@@ -255,7 +267,11 @@ static int run_aesccm128_tls(void *args)
     }
 
     /* Initialise the context for decryption with 16-byte key and 12-byte IV. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_128_ccm(), NULL, NULL, NULL);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_128_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed: ret %d\n",
              __func__, ret);
@@ -280,7 +296,11 @@ static int run_aesccm128_tls(void *args)
     }
 
     /* set 16-byte of key to the aes-ccm context. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit_ex(dec_ctx, NULL, NULL, key, NULL);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_128_ccm(), e, key, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed for setting key: ret %d\n", __func__, ret);
         goto err;
@@ -368,7 +388,9 @@ static int run_aesccm128_tls(void *args)
 static int run_aesccm128_update(void *args)
 {
     TEST_PARAMS *temp_args = (TEST_PARAMS *) args;
+#ifndef QAT_OPENSSL_PROVIDER
     ENGINE *e = temp_args->e;
+#endif
     int size = INPUT_TEXT_SIZE;
     int print_output = temp_args->print_output;
     int verify = temp_args->verify;
@@ -386,6 +408,7 @@ static int run_aesccm128_update(void *args)
     EVP_CIPHER_CTX *ctx = NULL;
     EVP_CIPHER_CTX *dec_ctx = NULL;
 
+#ifndef QAT_OPENSSL_PROVIDER
     if (e != NULL) {
         const EVP_CIPHER *c = ENGINE_get_cipher(e, NID_aes_128_ccm);
 
@@ -394,6 +417,7 @@ static int run_aesccm128_update(void *args)
             e = NULL;
         }
     }
+#endif
 
     /*
      * Set the plaintext input data.
@@ -420,7 +444,11 @@ static int run_aesccm128_update(void *args)
     }
 
     /* Initialize encryption context for aes-ccm */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit(ctx, EVP_aes_128_ccm(), NULL, NULL);
+#else
     ret = EVP_EncryptInit_ex(ctx, EVP_aes_128_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() encryption failed while setting context: ret %d\n", __func__, ret);
         goto err;
@@ -441,15 +469,21 @@ static int run_aesccm128_update(void *args)
         goto err;
     }
 
+#ifndef QAT_OPENSSL_PROVIDER
     /* set 16-byte of key to the aes-ccm context. */
     ret = EVP_EncryptInit_ex(ctx, NULL, NULL, key, NULL);
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed while setting key: ret %d\n", __func__, ret);
         goto err;
     }
+#endif
 
     /* set 12-byte of iv to the aes-ccm context. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit(ctx, NULL, key, iv);
+#else
     ret = EVP_EncryptInit_ex(ctx, NULL, NULL, NULL, iv);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed while setting iv: ret %d\n", __func__, ret);
         goto err;
@@ -510,7 +544,11 @@ static int run_aesccm128_update(void *args)
     }
 
     /* Initialize decryption context for aes-ccm */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit(dec_ctx, EVP_aes_128_ccm(), NULL, NULL);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_128_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed: ret %d\n",
              __func__, ret);
@@ -541,7 +579,11 @@ static int run_aesccm128_update(void *args)
      *  Initialise the aes-ccm decryption context with 16-byte key and
      *  12-byte IV.
      */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit(dec_ctx, NULL, key, iv);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, NULL, NULL, key, iv);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed: ret %d\n",
              __func__, ret);
