@@ -65,6 +65,8 @@
 
 #define AES_CBC_BLOCK_SIZE     64
 
+#define NUM_ENCRYPTION_ITERATIONS 6
+
 #define FAIL_MSG(fmt, args...)  WARN( "# FAIL " fmt, ##args)
 #define FAIL_MSG_END(fmt, args...)  INFO( "# FAIL " fmt, ##args)
 #define PASS_MSG(fmt, args...)  INFO( "# PASS " fmt, ##args)
@@ -358,8 +360,6 @@ static int perform_op(EVP_CIPHER_CTX *ctx, unsigned char **in,
 
     return 1;
 
-    OPENSSL_free(inb);
-
  err:
     return ret;
 }
@@ -518,6 +518,7 @@ static int encrypt_and_compare(const test_info *t, int *buflen)
     }
 
  err:
+    OPENSSL_free(textbuf);
     return ret;
 }
 
@@ -571,6 +572,7 @@ static int test_crypto_op(const test_info *t, int enc_imp, int dec_imp)
     ret = 1;
 
  err:
+    OPENSSL_free(textbuf);
     return ret;
 }
 
@@ -617,6 +619,7 @@ int test_no_hmac_key_set(const test_info *t)
 
  err:
     EVP_CIPHER_CTX_free(ctx);
+    OPENSSL_free(buf);
 
     return ret;
 }
@@ -634,9 +637,9 @@ int test_multi_op(const test_info *t)
     char msgstr[128];
     int i = 0;
     EVP_CIPHER_CTX *ctx = NULL;
-    unsigned char *buf[6] = { NULL };
-    unsigned char *ebuf[6] = { NULL };
-    int num_encbytes[6] = { 0 };
+    unsigned char *buf[NUM_ENCRYPTION_ITERATIONS] = { NULL };
+    unsigned char *ebuf[NUM_ENCRYPTION_ITERATIONS] = { NULL };
+    int num_encbytes[NUM_ENCRYPTION_ITERATIONS] = { 0 };
 
     /* str to append to message to distinguish test runs */
     snprintf(msgstr, 128, "[%s: multi op %s] ", t->c->name, t->tls->v_str);
@@ -650,7 +653,7 @@ int test_multi_op(const test_info *t)
     /* Compute the size of the buffer needed for this TLS use case */
     size = compute_buff_size(size, &ivlen, pad, t->tls->v, ctx);
 
-    for (i = 0; i < 6; i++) {
+    for (i = 0; i < NUM_ENCRYPTION_ITERATIONS; i++) {
         ret = perform_op(ctx, &buf[i], &ebuf[i], size,
                          &num_encbytes[i], t->tls->v);
         if (ret != 1) {
@@ -665,6 +668,9 @@ int test_multi_op(const test_info *t)
 
  err:
     EVP_CIPHER_CTX_free(ctx);
+    for (i = 0; i < NUM_ENCRYPTION_ITERATIONS; i++) {
+        OPENSSL_free(buf[i]);
+    }
     return ret;
 }
 
@@ -804,6 +810,8 @@ static int test_auth_header(const test_info *t, int impl)
 
 err:
     EVP_CIPHER_CTX_free(ctx);
+    OPENSSL_free(textbuf);
+
     return ret;
 }
 
@@ -852,6 +860,7 @@ static int test_auth_pkt(const test_info *t, int impl)
     ret = 1;
 
 err:
+    OPENSSL_free(textbuf);
     return ret;
 }
 #ifndef QAT_OPENSSL_PROVIDER
@@ -1133,6 +1142,9 @@ static int run_aes_cbc_hmac_sha(void *pointer)
         for (i = 0; i < ntls; i++) {
             ti.tls = (tls_v *)&test_tls[i];
 #ifdef QAT_OPENSSL_PROVIDER
+# ifdef QAT_INSECURE_ALGO
+    insecure_algorithms_enabled = 1;
+# endif
             if (
                 /*
                  * Running the test with SW implementation to check if
@@ -1142,28 +1154,37 @@ static int run_aes_cbc_hmac_sha(void *pointer)
                 (test_auth_header(&ti, USE_SW) != 1) ||
                 (test_auth_pkt(&ti, USE_SW) != 1) ||
                 (
-                    /*
-                    * Perform these tests only if engine
-                    * is present.
-                    */
-                    (test_encrypted_buffer(&ti) != 1) ||
-                    (test_no_hmac_key_set(&ti) != 1) ||
-                    (test_crypto_op(&ti, USE_ENGINE, USE_SW) != 1) ||
-                    (test_crypto_op(&ti, USE_SW, USE_ENGINE) != 1) ||
-                    (test_crypto_op(&ti, USE_ENGINE, USE_ENGINE) != 1) ||
-                    (test_auth_header(&ti, USE_ENGINE) != 1) ||
-                    (test_auth_pkt(&ti, USE_ENGINE) != 1) ||
-                    (test_multi_op(&ti) != 1)
-                    /*
-                    * 1. cipher pipeline feature is not support in
-                    *    qatprovider due to limitation.
-                    * 2. small package offloading is support in qat-
-                    *    provider, but there's no relevant API to set
-                    *    threshold in OpenSSL 3.0, so this testcase is
-                    *    also deactiaved.
-                    */
-                )
-               ) {
+		   /*
+		    * Perform these tests only if engine is present.
+		    *
+		    * When QAT insecure algorithms are disabled, AES128_CBC_HMAC_SHA256
+		    * implementation uses only OpenSSL.
+		    *
+		    * USE_SW refers OpenSSL implementation.
+		    * USE_ENGINE refers QAT_Engine implementation.
+		    */
+                    (insecure_algorithms_enabled ||
+		        args->type != TEST_AES128_CBC_HMAC_SHA256) &&
+                    (
+                        (test_encrypted_buffer(&ti) != 1) ||
+                        (test_no_hmac_key_set(&ti) != 1) ||
+                        (test_crypto_op(&ti, USE_ENGINE, USE_SW) != 1) ||
+                        (test_crypto_op(&ti, USE_SW, USE_ENGINE) != 1) ||
+                        (test_crypto_op(&ti, USE_ENGINE, USE_ENGINE) != 1) ||
+                        (test_auth_header(&ti, USE_ENGINE) != 1) ||
+                        (test_auth_pkt(&ti, USE_ENGINE) != 1)
+                    )
+                ) ||
+                (test_multi_op(&ti) != 1)
+                /*
+                 * 1. cipher pipeline feature is not support in
+                 *    qatprovider due to limitation.
+                 * 2. small package offloading is support in qat-
+                 *    provider, but there's no relevant API to set
+                 *    threshold in OpenSSL 3.0, so this testcase is
+                 *    also deactiaved.
+                 */
+            ) {
 #else
             if (
                 /*
