@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023-2024 Intel Corporation.
+ *   Copyright(c) 2023-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -134,7 +134,7 @@ static int run_sm4ccm_update(void *args)
     }
 #endif
 
-    if (plaintext == NULL || ciphertext == NULL || dec_cipher == NULL) {
+    if (ciphertext == NULL || dec_cipher == NULL) {
         INFO("# FAIL: [%s] --- Initial parameters malloc failed ! \n",
              __func__);
         exit(EXIT_FAILURE);
